import { Box3, Frustum, Raycaster, Vector3 } from "three";
import { Node } from "./Node";
import { Tree } from "./Tree";
/**
 * A pointer-based octree that subdivides space for fast spatial searches.
 */
export declare class Octree implements Tree, Iterable<Node> {
    /**
     * The root octant.
     */
    protected root: Node;
    /**
     * Constructs a new octree.
     *
     * @param root - The root node. See {@link Octant} or {@link CubicOctant}.
     */
    constructor(root: Node);
    get min(): Vector3;
    get max(): Vector3;
    get children(): Node[];
    getCenter(result: Vector3): Vector3;
    getDimensions(result: Vector3): Vector3;
    /**
     * Recursively collects nodes that intersect with the specified region.
     *
     * @param region - A region.
     * @return The nodes.
     */
    cull(region: Frustum | Box3): Node[];
    /**
     * Calculates the current depth of this octree.
     *
     * @return The depth.
     */
    getDepth(): number;
    /**
     * Fetches all nodes of a specific depth level.
     *
     * @param level - The depth level.
     * @return The nodes.
     */
    findNodesByLevel(level: number): Node[];
    /**
     * Finds nodes that intersect with the given ray. The intersecting nodes are
     * sorted by distance, closest first.
     *
     * @param raycaster - A raycaster.
     * @return The intersecting nodes.
     */
    getIntersectingNodes(raycaster: Raycaster): Node[];
    /**
     * Returns an iterator that traverses the octree and returns leaf nodes.
     *
     * When a cull region is provided, the iterator will only return leaves that
     * intersect with that region.
     *
     * @param region - A cull region.
     * @return An iterator.
     */
    leaves(region?: Frustum | Box3): Iterator<Node>;
    /**
     * Returns an iterator that traverses the octree and returns all leaf nodes.
     *
     * @return An iterator.
     */
    [Symbol.iterator](): Iterator<Node>;
}
